# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from airflow.providers.edge3.cli.dataclasses import MaintenanceMarker, WorkerStatus
from airflow.providers.edge3.models.edge_worker import EdgeWorkerState

MOCK_ENDPOINT = "https://invalid-api-test-endpoint"


class TestMaintenanceMarker:
    def test_maintenance_marker_json(self):
        marker = MaintenanceMarker(maintenance="maintenance", comments="comments")
        assert marker == MaintenanceMarker.from_json(marker.json)


class TestWorkerStatus:
    def test_worker_status_json(self):
        status = WorkerStatus(
            job_count=1,
            jobs=[],
            state=EdgeWorkerState.RUNNING,
            maintenance=False,
            maintenance_comments=None,
            drain=False,
        )
        assert status == WorkerStatus.from_json(status.json)
